/**
 * Helper
 * Create a chatbot with AI features for your website.
 * Exclusively on https://1.envato.market/helper
 *
 * @encoding        UTF-8
 * @version         1.0.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Cherviakov Vlad (vladchervjakov@gmail.com), Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmytro Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/
/**
 * Helper
 * Create a chatbot with AI features for your website.
 * Exclusively on https://1.envato.market/helper
 *
 * @encoding        UTF-8
 * @version         1.0.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Cherviakov Vlad (vladchervjakov@gmail.com), Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmytro Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

jQuery( function ( $ ) {

    "use strict";

    $( document ).ready( function () {
        const transitionDuration = 100;

        /**
         * Hide empty fields
         */
        function manageEmptyFields( action, fieldsClass, hideBefore, hideAfter, isTextarea = true ) {
            let showedTranslations = 20;

            const repeaterFields = $( `.${fieldsClass}` );

            repeaterFields.each( function() {

                let field = $( this ).val();

                if ( !isTextarea ) {
                    field = $( this ).children( 'input' ).val();
                }

                // Hide all groups with empty fields
                if ( field === '' ) {

                    // Hide field-group with empty field
                    if ( action === 'hide' ) {
                        $( this ).closest( 'tr' ).prevAll().slice( 0, hideBefore ).hide( transitionDuration );
                        $( this ).closest( 'tr' ).nextAll().slice( 0, hideAfter ).hide( transitionDuration );
                        $( this ).closest( 'tr' ).hide();

                        showedTranslations--;

                    }

                    // Show field-group with empty field
                    if ( action === 'show' ) {

                        $( this ).closest( 'tr' ).prevAll().slice( 0, hideBefore ).show( transitionDuration );
                        $( this ).closest( 'tr' ).nextAll().slice( 0, hideAfter ).show( transitionDuration );
                        $( this ).closest( 'tr' ).show();

                        showedTranslations++;

                        return false;

                    }

                }

            } );

        }

        /**
         * Calculate showing or hided fields
         * @param action
         * @param fieldsClass
         * @returns {number}
         */
        function calculateRepeaterFields( action, fieldsClass ) {

            const repeaterFields = $( `.${fieldsClass}` );
            let showingFields = 0;
            let hidedFields = 0;

            repeaterFields.each( function () {

                $( this ).closest( 'tr' ).css( 'display' ) !== 'none' ?
                    showingFields++ :
                    hidedFields++;

            } );

            if ( action === 'show' ) {

                return showingFields;

            } else if ( action === 'hide' ) {

                return hidedFields;

            } else {

                return 0;

            }

        }

        /**
         * Show or hide buttons if possible to show or hide something
         */
        function manageButtons( fieldsClass, removeButtonId, addButtonId ) {

            const $addButton = $( `#${addButtonId}` );
            const $removeButton = $( `#${removeButtonId}` );

            // Hide button if nothing to show
            calculateRepeaterFields( 'show', fieldsClass ) >= 20 ?
                $addButton.hide( 0 ) : $addButton.show( 0 );

            // Hide button if nothing to hide
            calculateRepeaterFields( 'hide', fieldsClass ) >= 20 ?
                $removeButton.hide( 0 ) : $removeButton.show( 0 );

        }

        /**
         * Check repeater fields and show notices for no=empty fields
         */
        function notEmptyFieldNotice( fieldsClass, fieldName ) {

            const repeaterFields = $( `.${fieldsClass}` );
            let hideEmpty = false;
            let $targetField = false;

            repeaterFields.each( function() {

                const field = $( this ).val();
                const isVisible = $( this ).parent().parent().parent().css( 'display' ) !== 'none';

                if ( field === '' && isVisible ) {

                    hideEmpty = true;

                }

                // Find last filled field
                if ( field !== '' && isVisible ) {

                    $targetField= $( this );

                }


            } );

            if ( !hideEmpty ) {

                addNotice( fieldName,'warning', 5000 );

                $targetField.addClass( 'mdc-warning-field' );

                // Make filed focused and looks like focused
                $targetField.addClass( 'mdc-text-field--focused' );
                $targetField.focus();

                // Bind remove action to button
                $( `#mdp-repeater-field-notice-${fieldName} button` ).on( 'click', () => {

                    $targetField.val( '' );
                    $( '#submit' ).click();

                } );

            }

        }

        /** Add notice for repeater field
         *
         * @param fieldName
         * @param design
         * @param timeout
         */
        function addNotice( fieldName, design= 'info', timeout = 5000 ) {

            // Show only one notice
            if ( $( `#mdp-repeater-field-notice-${fieldName}` ).length > 0 ) { return; }

            $( '#wpbody-content form .wrap' ).append( `
<div id="mdp-repeater-field-notice-${fieldName}" class="mdc-snackbar mdc-${design} mdc-snackbar--open" data-timeout="${timeout}" data-mdc-index="1">
    <div class="mdc-snackbar__surface">
        <div class="mdc-snackbar__label" role="status" aria-live="polite">Clear the ${fieldName} field before deleting the field</div>
        <div class="mdc-snackbar__actions">
            <button class="mdc-button mdc-snackbar__action" type="button" title="Clear field">Clear ${fieldName}</button>
        </div>
    </div>   
</div>
            ` );

            setTimeout( () => {

                $( `#mdp-repeater-field-notice-${fieldName}` ).remove();
                $( '.mdc-warning-field' ).removeClass( 'mdc-warning-field' );

            }, timeout + 10 );

        }

        /**
         * Init repeater.
         */
        /**
         * Init translation buttons
         */
        function initRepeaterButtons( addButtonId, removeButtonId, fieldsClass, fieldName, hideBefore, hideAfter, isTextarea = true ) {

            const $addButton = $( `#${addButtonId}` );
            $addButton.on( 'click', ( e ) => {

                e.preventDefault();
                manageEmptyFields( 'show', fieldsClass, hideBefore, hideAfter, isTextarea );
                manageButtons( fieldsClass, removeButtonId, addButtonId );

            } );

            const $removeButton = $( `#${removeButtonId}` );
            $removeButton.detach();
            $addButton.parent().append( $removeButton );

            $removeButton.on( 'click', ( e ) => {

                e.preventDefault();
                notEmptyFieldNotice( fieldsClass, fieldName );
                manageEmptyFields( 'hide', fieldsClass, hideBefore, hideAfter, isTextarea );
                manageButtons( fieldsClass, removeButtonId, addButtonId );

            } );

        }

        /**
         * Hide or close next tr after switch
         * @param $element
         * @param num
         */
        function switchSingle( $element, num ) {

            for ( let i = 0; i < num; i++ ) {

                $element.is( ':checked' ) ?
                    $element.closest( 'tr' ).nextAll( 'tr' ).eq( i ).show( 300 ) :
                    $element.closest( 'tr' ).nextAll( 'tr' ).eq( i ).hide( 300 );

            }

        }

        /**
         * Init single switch
         * @param $element
         * @param num
         */
        function initSingleSwitch( $element, num = 1 ) {

            switchSingle( $element, num );

            $element.on( 'change', () => {

                switchSingle( $element, num );

            } );

        }

        function initDropZone( key_id, drop_zone_id, type ) {
            /** Show Error message under drop zone. */
            let $dropZone = $( `.mdp-tab-name-general #${drop_zone_id}` );

            $dropZone.on( 'dragenter', function() {
                hideMessage();
                $( this ).addClass( 'mdp-hover' );
            } );

            $dropZone.on('dragleave', function() {
                $( this ).removeClass( 'mdp-hover' );
            } );

            /** Text Input to store key file. */
            let $key_input = $( `.mdp-tab-name-general #${key_id}` );

            /** Setup Drag & Drop. */
            $dropZone.on( 'dragover', handleDragOver );

            /**
             * Read dragged file by JS.
             **/
            $dropZone.on( 'drop', function ( e ) {

                e.stopPropagation();
                e.preventDefault();

                // Show busy spinner.
                $( this ).removeClass( 'mdp-hover' );
                $dropZone.addClass( 'mdp-busy' );

                let file = e.originalEvent.dataTransfer.files[0]; // FileList object.

                /** Check is one valid JSON file. */
                if ( ! checkSoundFile( file ) ) {
                    $dropZone.removeClass( 'mdp-busy' );
                    return;
                }

                /** Read key file to input. */
                readFile( file )

            } );

            /**
             * Read key file to input.
             **/
            function readFile( file ) {

                let reader = new FileReader();
                const $submitButton = $( '#submit' );

                /** Closure to capture the file information. */
                reader.onload = ( function( theFile ) {

                    return function( e ) {

                        const fileName = type + '.' + theFile.name.split( '.' )[1];

                        $key_input.attr( 'value', fileName );

                        /** Hide error messages. */
                        hideMessage();

                        /** If we have valid sound. */
                        if ( $key_input.attr( 'value' ).length > 0 ) {

                            const blob = new Blob( [e.target.result], { type: theFile.type } );

                            const xhr = new XMLHttpRequest();
                            const formData = new FormData();
                            xhr.open( 'POST', mdpHelperUnity.ajaxURL, true );
                            formData.append( 'nonce', mdpHelperUnity.nonce );
                            formData.append( 'action', 'mdp_helper_save_sound' );
                            formData.append( 'mdp_sound_file_name', fileName );
                            formData.append( 'mdp_sound_file', blob );
                            formData.append( 'mdp_sound_type', type );
                            xhr.onload = () => {
                                if ( xhr.status === 200 ) {
                                    $submitButton.click(); // Save settings.
                                } else {
                                    console.log( 'Error occurred while saving file!' )
                                }
                            };
                            xhr.onerror = () => {

                                // Error sending request
                                console.error('Error sending request!');

                            };
                            xhr.send( formData );


                        } else {

                            showErrorMessage( 'Error: Failed to read file. Please try again.' );

                            $dropZone.removeClass( 'mdp-busy' );

                        }

                    };

                } )( file );

                /** Read file as text. */
                reader.readAsArrayBuffer( file );

            }

            /**
             * Show upload form on click.
             **/
            let $file_input = $dropZone.next().next();
            $dropZone.on( 'click', function () {
                $file_input.click();
            } );

            $file_input.on( 'change', function ( e ) {

                $dropZone.addClass( 'mdp-busy' );

                let file = e.target.files[0];

                /** Check is one valid JSON file. */
                if ( ! checkSoundFile( file ) ) {
                    $dropZone.removeClass( 'mdp-busy' );
                    return;
                }

                /** Read key file to input. */
                readFile( file );

            } );

            function showErrorMessage( msg ) {

                let $msgBox = $dropZone.next();

                $msgBox.addClass( 'mdp-error' ).html( msg );

            }

            function handleDragOver( e ) {

                e.stopPropagation();
                e.preventDefault();

            }

            /** Hide message under drop zone. */
            function hideMessage() {

                let $msgBox = $dropZone.next();

                $msgBox.removeClass( 'mdp-error' ).html( '' );

            }

            /**
             * Check file is a single valid MP3 file.
             *
             * @param file - MP3 file to check.
             **/
            function checkSoundFile( file ) {

                /** Select only one file. */
                if ( null == file ) {

                    showErrorMessage( 'Error: Failed to read file. Please try again.' );

                    return false;

                }


                /** Process json file only. */
                if ( ! file.type.match( 'audio/mpeg' ) && ! file.type.match( 'audio/wav' ) && ! file.type.match( 'audio/ogg' ) ) {

                    showErrorMessage( 'Error: File must have valid audio format.' );

                    return false;

                }

                return true;
            }

            /** Reset Key File. */
            $dropZone.next().find( '.mdp-reset-key-btn' ).on( 'click', function () {
                const xhr = new XMLHttpRequest();
                const formData = new FormData();
                xhr.open( 'POST', mdpHelperUnity.ajaxURL, true );
                formData.append( 'nonce', mdpHelperUnity.nonce );
                formData.append( 'action', 'mdp_helper_remove_audio' );
                formData.append( 'mdp_sound_type', type );
                formData.append( 'mdp_file_name', $key_input.attr( 'value' ) );

                xhr.onload = () => {
                    if ( xhr.status === 200 ) {
                        $key_input.val( '' );
                        $( '#submit' ).trigger( 'click' );
                    } else {
                        console.log( 'Error occurred while resetting file!' )
                    }
                };
                xhr.onerror = () => {
                    // Error sending request
                    console.error( 'Error sending request!' );

                };
                xhr.send( formData );

            } );

        }

        /** Single select */
        function resetFields( fieldsGroupClass ) {
            $( `.${fieldsGroupClass}` ).closest( 'tr' ).hide( 300 );
        }

        function showBasedOnSelect( $select, fieldsGroupClass, fieldId ) {
            const $listItems = $select.closest( 'tr' ).find( '.mdc-list-item' );

            resetFields( fieldsGroupClass );

            $( `.${fieldId + $select.val() }` ).closest( 'tr' ).show( 300 );

            $listItems.on( 'click', ( e ) => {
                resetFields( fieldsGroupClass );
                $( `.${fieldId + e.target.dataset.value }` ).closest( 'tr' ).show( 300 );
            } );

        }

        /** Show based on select in repeater */
        function showBasedOnSelectInRepeater( fieldName, fieldId, fieldGroup, repeaterSize ) {
            for ( let i = 0; i < repeaterSize; i++ ) {
                showBasedOnSelect( $( `#${fieldName}_${i}` ), `${fieldGroup}-${i}`, `${fieldId}-${i}-` );
            }

        }

        function showBasedOnChoices( selectId, fieldsGroupClass, fieldClass ) {
            const $choicesSelect = $( `#${selectId}` );

            resetFields( fieldsGroupClass );

            const choices = $choicesSelect.val();

            if ( !choices ) { return; }

            choices.forEach( choice => {
                $( `.${fieldClass + choice}` ).closest( 'tr' ).show( 300 );
            } );

            $choicesSelect.on( 'change', function () {
              resetFields( fieldsGroupClass );
              const choices = $( this ).val();

              choices.forEach( choice => {
                $( `.${fieldClass + choice}` ).closest( 'tr' ).show( 300 );
              } );

            } );

        }

        /**
         * Show avatar fields based on selected value.
         * @param type
         */
        function showAvatarFields( type ) {

            const triggerInput = $( `#mdp_helper_avatar_settings_${ type }_avatar` );
            if ( ! triggerInput.length ) { return; }

            const avatarImage = $( `#mdp_helper_avatar_settings_${ type }_avatar_image` ).closest( 'tr' );
            const avatarIcon = $( `#mdp_helper_avatar_settings_${ type }_avatar_icon` ).closest( 'tr' );
            const avatarColor = $( `#mdp_helper_avatar_settings_${ type }_avatar_color` ).closest( 'tr' );
            const avatarBackground = $( `#mdp_helper_avatar_settings_${ type }_avatar_background` ).closest( 'tr' );

            // Init
            avatarFields( avatarImage, avatarIcon, avatarColor, avatarBackground, triggerInput.val() );

            // On change
            triggerInput.on( 'change', function () {

                avatarFields( avatarImage, avatarIcon, avatarColor, avatarBackground, this.value );
                console.log( this.value );

            } );

        }

        /**
         * Show avatar fields based on selected value.
         * @param avatarImage
         * @param avatarIcon
         * @param avatarColor
         * @param avatarBackground
         * @param value
         */
        function avatarFields( avatarImage, avatarIcon, avatarColor, avatarBackground, value ) {

            console.log( value );

            switch ( value ) {

                case 'icon':

                    avatarImage.hide( 0 );
                    avatarIcon.show( 300 );
                    avatarColor.show( 300 );
                    avatarBackground.show( 300 );
                    break;

                case 'image':

                    avatarImage.show( 300 );
                    avatarIcon.hide( 0 );
                    avatarColor.hide( 300 );
                    avatarBackground.show( 300 );
                    break;

                case 'none':

                    avatarIcon.hide();
                    avatarImage.hide();
                    avatarColor.hide();
                    avatarBackground.hide();
                    break;

                default:

                    return;

            }

        }

        /**
         * Show box fields
         * @param $element
         * @param value
         */
        function showBoxShadowFields( $element, value ) {

            boxShadowFields( $element );

            $element.on( 'change', () => {

                boxShadowFields( $element );

            } );

        }

        /**
         * Show box shadow fields based on selected value.
         * @param $element
         */
        function boxShadowFields( $element ) {

            if ( $element.val() === 'none' ) {

                $element.closest( 'tr' ).next().hide();
                $element.closest( 'tr' ).next().next().hide();

            } else {

                $element.closest( 'tr' ).next().show( 300 );
                $element.closest( 'tr' ).next().next().show( 300 );

            }

        }

        /**
         * Float button tab
         */
        function uiFloatButtonTab() {

            // Master switch
            const $floatButtonSwitch = $( '#mdp_helper_float_button_settings_open_bot_with_button' );

            // Init
            if ( $floatButtonSwitch.prop('checked') ) {

                initSingleSwitch( $( '#mdp_helper_float_button_settings_open_bot_button_enable_icon' ), 4 );
                initSingleSwitch( $( '#mdp_helper_float_button_settings_open_bot_button_enable_caption' ), 4 );

            } else {

                switchSingle( $floatButtonSwitch, 27 );

            }

            // On change
            $floatButtonSwitch.on( 'change', () => {

                switchSingle( $floatButtonSwitch, 27 );

                if ( $floatButtonSwitch.prop('checked')  ) {

                    switchSingle( $floatButtonSwitch, 13 );

                    switchSingle( $( '#mdp_helper_float_button_settings_open_bot_button_enable_icon' ), 4 );
                    switchSingle( $( '#mdp_helper_float_button_settings_open_bot_button_enable_caption' ), 4 );

                }

            } );

        }

        /**
         * Send button tab
         */
        function uiSendButtonTab() {

            const $sendButtonSwitch = $( '#mdp_helper_form_settings_send_button_show' );

            if ( ! $sendButtonSwitch.prop('checked') ) {

                setTimeout( () => {

                    initSingleSwitch( $sendButtonSwitch, 16 );

                }, 100 );

            } else {

                initSingleSwitch( $sendButtonSwitch, 16 );

            }

        }

        /**
         * Upper line fields group
         */
        function uiSignature() {

            const $ulNameSwitcher = $( '#mdp_helper_message_settings_upper_line_name_enabled' );
            const $ulTimestampSwitcher = $( '#mdp_helper_message_settings_upper_line_timestamp_enabled' );
            const $ulButtonsSwitcher = $( '#mdp_helper_message_settings_upper_line_buttons_enabled' );

            initSingleSwitch( $ulNameSwitcher, 1 );
            initSingleSwitch( $ulTimestampSwitcher, 1 );
            initSingleSwitch( $ulButtonsSwitcher, 2 );

            // Init
            uiSignatureConditionFields( $ulNameSwitcher, $ulTimestampSwitcher, $ulButtonsSwitcher );

            // On change
            $( '#mdp_helper_message_settings_upper_line_name_enabled, #mdp_helper_message_settings_upper_line_timestamp_enabled, #mdp_helper_message_settings_upper_line_buttons_enabled' ).on( 'change', () => {

                uiSignatureConditionFields( $ulNameSwitcher, $ulTimestampSwitcher, $ulButtonsSwitcher );

            } );

        }

        /**
         * Show or hide upper line fields based on selected values.
         * @param $ulNameSwitcher
         * @param $ulTimestampSwitcher
         * @param $ulButtonsSwitcher
         */
        function uiSignatureConditionFields( $ulNameSwitcher, $ulTimestampSwitcher, $ulButtonsSwitcher ) {

            const $uiColor = $( '#mdp_helper_message_settings_upper_line_color' );
            const $uiFontSize = $( '#mdp_helper_message_settings_upper_line_font_size' );

            if ( ! $ulButtonsSwitcher.prop( 'checked' ) && ! $ulTimestampSwitcher.prop( 'checked' ) && ! $ulNameSwitcher.prop( 'checked' ) ) {

                $uiColor.closest( 'tr' ).hide( 0 );
                $uiFontSize.closest( 'tr' ).hide( 0 );

            } else {

                $uiColor.closest( 'tr' ).show( 300 );
                $uiFontSize.closest( 'tr' ).show( 300 );

            }

        }

        function getPosts( postTypes ) {
            const xhr = new XMLHttpRequest();
            const formData = new FormData();
            const url = new URL( mdpHelperUnity.ajaxURL );
            url.searchParams.set( 'action', 'mdp_helper_get_posts' );
            url.searchParams.set( 'mdp_helper_post_types', postTypes );
            url.searchParams.set( 'mdp_helper_nonce', mdpHelperUnity.nonce  );

            return new Promise( ( resolve, reject ) => {
                xhr.open( 'GET', url, true );
                xhr.onload = () => resolve( xhr.responseText );
                xhr.onerror = () => reject( xhr.statusText );
                xhr.send( formData );
            } );
        }

        function createChoicesOptions( postsData ) {
            let options = '';

            for ( let postId in postsData.posts ) {
                options += `<option ${postsData.selected_posts.includes( postId ) ? 'selected=""' : ''} value="${postId}">
                                ${postsData.posts[postId]}
                            </option>`;
            }

            return options;
        }

        function createNewSelect ( options ) {
            return `<select 
                    id="mdp_helper_ai_settings_open_ai_post_id" 
                    name="mdp_helper_ai_settings[open_ai_post_id][]" 
                    multiple
                    data-placeholder
                    class="mdp-chosen chosen-select"
                    style="display: none;">${options}</select>`;
        }

        function updatePostIdControl() {
            const $choicesSelect = $( "#mdp_helper_ai_settings_open_ai_post_types" );

            $choicesSelect.on( 'change', async function () {
                const posts = await getPosts( $choicesSelect.val() );
                const postsData = JSON.parse( posts ).data;

                const $postsIdsSelect = $( '#mdp_helper_ai_settings_open_ai_post_id' );
                const $choicesContainer = $( "#mdp_helper_ai_settings_open_ai_post_id_chosen" );

                const $container = $postsIdsSelect.parent();

                /** Remove old chosen containers and select */
                $postsIdsSelect.remove();
                $choicesContainer.remove();

                $container.append( createNewSelect( createChoicesOptions( postsData ) ) );


                /** Init new chosen */
                const updatedPostIdsSelect = $container.find( '#mdp_helper_ai_settings_open_ai_post_id' );

                updatedPostIdsSelect.chosen( {
                    width: '100%',
                    search_contains: true,
                    disable_search_threshold: 7,
                    inherit_select_classes: true,
                    no_results_text: 'Oops, nothing found',
                    allow_single_deselect: true,
                } );

            } );
        }

        /**
         * Init meta-boxes user interface
         */
        function initUI() {

            /** Updates post id control on post types control change */
            updatePostIdControl();

            /** Show controls based on selected prompt type */
            showBasedOnChoices( 'mdp_helper_ai_settings_open_ai_prompt_type', 'mdp-helper-open-ai-fields', 'mdp-helper-prompt-' );

            /** Show animations controls based on select */
            showBasedOnSelect( $( '#mdp_helper_popup_settings_bot_container_animation' ), 'mdp-helper-popup-animation', 'mdp-helper-tab-animation-type-' );
            showBasedOnSelect( $( '#mdp_helper_message_settings_bot_message_animation' ), 'mdp-helper-bot-message-animation', 'mdp-helper-bot-message-animation-type-' );
            showBasedOnSelect( $( '#mdp_helper_message_settings_user_message_animation' ), 'mdp-helper-user-message-animation', 'mdp-helper-user-message-animation-type-' );

            /** Show based on select */
            showBasedOnSelectInRepeater( 'mdp_helper_collect_data_settings_collect_data_field_validation', 'mdp-helper-validation', 'mdp-helper-collect-data-fields', 20 );

            /** Show controls based on selected bot type */
            showAvatarFields( 'bot' );
            showAvatarFields( 'user' );

            /** Chat header content */
            initSingleSwitch( $( '#mdp_helper_header_settings_chat_header_heading' ), 3 );
            initSingleSwitch( $( '#mdp_helper_header_settings_close_button_show' ), 6 );

            /** Collect data */
            initSingleSwitch( $( '#mdp_helper_collect_data_settings_send_user_data_email' ), 1 );

            /** Chat footer content */
            initSingleSwitch( $( '#mdp_helper_footer_settings_chat_footer_text_show' ), 3 );

            /** Open button */
            uiFloatButtonTab();

            /** Send button */
            uiSendButtonTab();

            /** Behaviour */
            initSingleSwitch( $( '#mdp_helper_behavior_settings_bot_stt' ), 4 );
            initSingleSwitch( $( '#mdp_helper_behavior_settings_bot_tts' ), 2 );
            initSingleSwitch( $( '#mdp_helper_behavior_settings_bot_typing_animation' ), 1 );
            initSingleSwitch( $( '#mdp_helper_behavior_settings_bot_respond_delay_enabled' ), 3 );
            initSingleSwitch( $( '#mdp_helper_behavior_settings_bot_memory_enabled' ), 1 );

            /** Signature */
            uiSignature();

            /** Toolbar */
            initSingleSwitch( $( '#mdp_helper_toolbar_settings_close_button_show' ), 6 );
            initSingleSwitch( $( '#mdp_helper_toolbar_settings_bot_commands_enable' ), 2 );
            initSingleSwitch( $( '#mdp_helper_toolbar_settings_mute_button_enable' ), 2 );
            initSingleSwitch( $( '#mdp_helper_toolbar_settings_mail_button_enable' ), 3 );
            initSingleSwitch( $( '#mdp_helper_toolbar_settings_call_button_enable' ), 3 );
            initSingleSwitch( $( '#mdp_helper_toolbar_settings_messenger_button_enable' ), 3 );
            initSingleSwitch( $( '#mdp_helper_toolbar_settings_social_button_enable' ), 3 );

            const uiSelectsValueHolder = $( `.mdc-select input[type="hidden"]` );
            /** Show box shadow fields */
            uiSelectsValueHolder.each( function () {

                // If id ends with _border_style
                if ( $( this ).attr( 'id' ).endsWith( '_border_style' ) ) {

                    showBoxShadowFields( $( this ) );

                }

            } );

            /** Box shadow fields */
            uiSelectsValueHolder.each( function () {

                // If id ends with _border_style
                if ( $( this ).attr( 'id' ).endsWith( '_box_shadow' ) ) {

                    showBoxShadowFields( $( this ) );

                }

            } );

            /** Init send message sound file drop zone */
            initDropZone( 'mdp-helper-settings-dnd-send_message_sound', 'mdp-send_message_sound-drop-zone', 'send_message_sound' );

            /** Init receive message sound file drop zone */
            initDropZone( 'mdp-helper-settings-dnd-receive_message_sound', 'mdp-receive_message_sound-drop-zone', 'receive_message_sound' );

            /** Show controls depending switch value */
            initSingleSwitch( $( '#mdp_helper_design_settings_bot_typing_animation' ), 1 );
            initSingleSwitch( $( '#mdp_helper_open_bot_button_settings_open_bot_with_button' ), 15 );
            initSingleSwitch( $( '#mdp_helper_faq_settings_faq_pagination' ), 1 );

            /** Initial message repeater */
            initRepeaterButtons( 'mdp-add-message-general', 'mdp-remove-message-general', 'mdp-helper-general-message', 'message', 1, 1 );
            manageEmptyFields( 'hide', 'mdp-helper-general-message', 1 , 1 );
            manageButtons( 'mdp-helper-general-message', 'mdp-remove-message-general', 'mdp-add-message-general' );

            /** FAQ questions repeaters */
            initRepeaterButtons( 'mdp-add-faq', 'mdp-remove-faq', 'mdp-helper-faq-question', 'question', 1, 2 );
            manageEmptyFields( 'hide', 'mdp-helper-faq-question', 1 , 2 );
            manageButtons( 'mdp-helper-faq-question', 'mdp-remove-faq', 'mdp-add-faq' );

            initRepeaterButtons( 'mdp-add-message-faq', 'mdp-remove-message-faq', 'mdp-helper-faq-message', 'message', 1, 1 );
            manageEmptyFields( 'hide', 'mdp-helper-faq-message', 1, 1 );
            manageButtons( 'mdp-helper-faq-message', 'mdp-remove-message-faq', 'mdp-add-message-faq' );

            /** Collect user data success/error messages repeaters */
            initRepeaterButtons( 'mdp-add-message-collect_data', 'mdp-remove-message-collect_data', 'mdp-helper-collect_data-message', 'message', 1,1 );
            manageEmptyFields( 'hide', 'mdp-helper-collect_data-message', 1, 1 );
            manageButtons( 'mdp-helper-collect_data-message', 'mdp-remove-message-collect_data', 'mdp-add-message-collect_data' );


            /** Collect user data repeaters */
            initRepeaterButtons( 'mdp-add-collect-data', 'mdp-remove-collect-data', 'mdp-helper-collect-data-bot-message', 'message', 1,6, false );
            manageEmptyFields( 'hide', 'mdp-helper-collect-data-bot-message', 1, 6, false );
            manageButtons( 'mdp-helper-collect-data-bot-message', 'mdp-remove-collect-data', 'mdp-add-collect-data' );

            /** Init show based on select for user data repeater an add message button click */
            const $addButton = $( '#mdp-add-collect-data' );
            $addButton.on( 'click', () => {
                showBasedOnSelectInRepeater( 'mdp_helper_collect_data_settings_collect_data_field_validation', 'mdp-helper-validation', 'mdp-helper-collect-data-fields', 20 );
            } );


            /** Get user emails success/error messages repeaters */
            initRepeaterButtons( 'mdp-add-message-get_emails', 'mdp-remove-message-get_emails', 'mdp-helper-get_emails-message', 'message', 1,1 );
            manageEmptyFields( 'hide', 'mdp-helper-get_emails-message', 1, 1 );
            manageButtons( 'mdp-helper-get_emails-message', 'mdp-remove-message-get_emails', 'mdp-add-message-get_emails' );

            /** Get user emails repeaters */
            initRepeaterButtons( 'mdp-add-get-emails', 'mdp-remove-get-emails', 'mdp-helper-get-emails-bot-message', 'message', 1,1, false );
            manageEmptyFields( 'hide', 'mdp-helper-get-emails-bot-message', 1, 1, false );
            manageButtons( 'mdp-helper-get-emails-bot-message', 'mdp-remove-get-emails', 'mdp-add-get-emails' );

            /** Remove empty rows in settings table */
            $( '.mdp-tab-content table.form-table td' ).each( function () {

                if ( $( this ).children().length === 0 ) {
                    $( this ).parent().remove();
                }

            } );

        }

        initUI();



    } );

} );


