<?php
/**
 * Helper
 * Create a chatbot with AI features for your website.
 * Exclusively on https://1.envato.market/helper
 *
 * @encoding        UTF-8
 * @version         1.0.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Cherviakov Vlad (vladchervjakov@gmail.com), Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmytro Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Helper;

use Merkulove\Helper\Unity\Plugin;
use Merkulove\Helper\Unity\Settings;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * @package Merkulove\Helper
 **/
final class TabGlobal {

	/**
	 * Controls.
	 * @return array[]
	 */
	public static function controls(): array {

		$tabs = Plugin::get_tabs();

		$fields = array_merge(
			self::fields_general(),
		);

		$tabs[ 'general' ][ 'fields' ] = $fields;
		$tabs[ 'general' ][ 'icon' ] = 'settings';

		$tabs = Config::get_instance()->set_bot_response_messages(
			$tabs,
			'general',
			true,
			false,
			[
				'add_button' => esc_html__( 'Add initial message', 'helper' ),
				'remove_button' => esc_html__( 'Remove initial message', 'helper' )
			],
			[
				'bot_message' => esc_html__( 'Hi! How can i help you?', 'helper' ),
				'error_message' => esc_html__( "Sorry, I don't understand this", 'helper' )
			]
		);

		Plugin::set_tabs( $tabs );
		Settings::get_instance()->get_options();

		return $fields;

	}

	/**
	 * Fields general.
	 *
	 * @return array[]
	 */
	private static function fields_general(): array {

		return [

			'first_divider' => [
				'type'              => 'divider',
			],

			'first_heading' => [
				'type'              => 'header',
				'label'             => esc_html__( 'Global settings', 'helper' ),
				'description'       => esc_html__( '', 'helper' ),
			],

			'bot_features' => [
				'type'              => 'chosen',
				'label'             => esc_html__( 'Bot features', 'helper' ),
				'description'       => esc_html__( 'Select one or more bot features to be used', 'helper' ),
				'default'           => [ 'faq', 'collect_data', 'get_user_email', 'bot_commands' ],
				'options'           => [
					'faq' => esc_html__( 'FAQ', 'helper' ),
					'collect_data'  => esc_html__( 'Collect user data', 'helper' ),
					'get_user_email'  => esc_html__( 'Get user emails', 'helper' ),
					'bot_commands' => esc_html__( 'Bot commands', 'helper' ),
					'ai' => 'Open AI',
				],
				'attr' => [
					'class' => 'mdp-helper-bot-type-bot_basic mdp-helper-general-fields',
					'multiple' => 'multiple'
				]
			],

			'bot_position' => [
				'type'              => 'select',
				'label'             => esc_html__( 'Bot position', 'helper' ),
				'description'       => esc_html__( 'Choose bot position or use shortcode', 'helper' ) . ' [helper]',
				'default'           => 'fixed-bottom-right',
				'options'           => [
					'fixed-top-left' => esc_html__( 'Fixed top left', 'helper' ),
					'fixed-top-center' => esc_html__( 'Fixed top centre', 'helper' ),
					'fixed-top-right' => esc_html__( 'Fixed top right', 'helper' ),
					'fixed-left-center' => esc_html__( 'Fixed left centre', 'helper' ),
					'fixed-right-center' => esc_html__( 'Fixed right centre', 'helper' ),
					'fixed-bottom-left' => esc_html__( 'Fixed bottom left', 'helper' ),
					'fixed-bottom-center' => esc_html__( 'Fixed bottom centre', 'helper' ),
					'fixed-bottom-right' => esc_html__( 'Fixed bottom right', 'helper' ),
					'shortcode' => esc_html__( 'Shortcode', 'helper' ),
				],
			],

			'send_message_sound' => [
				'type'              => 'custom_type',
				'render'            => [ Config::get_instance(), 'render_import_file' ],
				'label'             => esc_html__( 'Send message sound(WAV, MP3, OGG)', 'helper' ),
				'show_label'        => true,
				'description'       => '',
				'show_description'  => false,
				'default'           => '',
			],

			'receive_message_sound' => [
				'type'              => 'custom_type',
				'render'            => [ Config::get_instance(), 'render_import_file' ],
				'label'             => esc_html__( 'Receive message sound(WAV, MP3, OGG)', 'helper' ),
				'show_label'        => true,
				'description'       => '',
				'show_description'  => false,
				'default'           => '',
			],

		];

	}

}

